@@ -30,19 +30,26 @@
 #include "comments.h"
 
 
+#define MAX_COMMENT_LEN 60
+
+
 FileData *
-file_data_new (const gchar *path, 
+file_data_new (const char       *path, 
 	       GnomeVFSFileInfo *info)
 {
 	FileData *fd;
 
 	fd = g_new (FileData, 1);
+
+	fd->ref = 1;
 	fd->path = g_strdup (path);
-	fd->name = g_strdup (info->name);
+	fd->name = file_name_from_path (fd->path);
+	fd->utf8_name = g_filename_to_utf8 (fd->name, -1, 0, 0, 0);
 	fd->size = info->size;
 	fd->ctime = info->ctime;
 	fd->mtime = info->mtime;
 	fd->error = FALSE;
+	fd->thumb = FALSE;
 	fd->comment = g_strdup ("");
 
 	return fd;
@@ -53,8 +60,8 @@ void
 file_data_update (FileData *fd)
 {
 	GnomeVFSFileInfo *info;
-	GnomeVFSResult result;
-	gchar *escaped;
+	GnomeVFSResult    result;
+	char             *escaped;
 
 	g_return_if_fail (fd != NULL);
 
@@ -71,6 +78,11 @@ file_data_update (FileData *fd)
 		return;
 	}
 
+	fd->name = file_name_from_path (fd->path);
+
+	g_free (fd->utf8_name);
+	fd->utf8_name = g_filename_to_utf8 (fd->name, -1, 0, 0, 0);
+
 	fd->size = info->size;
 	fd->mtime = info->mtime;
 	fd->ctime = info->ctime;
@@ -86,13 +98,8 @@ file_data_set_path (FileData *fd,
 	g_return_if_fail (fd != NULL);
 	g_return_if_fail (path != NULL);
 
-	if (fd->path)
-		g_free (fd->path);
-	if (fd->name)
-		g_free (fd->name);
-
+	g_free (fd->path);
 	fd->path = g_strdup (path);
-	fd->name = g_strdup (file_name_from_path (path));
 
 	file_data_update (fd);
 }
@@ -115,7 +122,7 @@ file_data_update_comment (FileData *fd)
 		return;
 	}
 
-	fd->comment = comments_get_comment_as_string (data);
+	fd->comment = comments_get_comment_as_string (data, "\n", "\n");
 	if (fd->comment == NULL)
 		fd->comment = g_strdup ("");
 
@@ -123,11 +130,35 @@ file_data_update_comment (FileData *fd)
 }
 
 
-void 
-file_data_free (FileData *fd)
+void
+file_data_ref (FileData *fd)
 {
-	g_free (fd->name);
-	g_free (fd->path);
-	g_free (fd->comment);
-	g_free (fd);
+	g_return_if_fail (fd != NULL);
+	fd->ref++;
+}
+
+
+void
+file_data_unref (FileData *fd)
+{
+	g_return_if_fail (fd != NULL);
+
+	fd->ref--;
+
+	if (fd->ref == 0) {
+		g_free (fd->path);
+		g_free (fd->utf8_name);
+		g_free (fd->comment);
+		g_free (fd);
+	}
+}
+
+
+void
+file_data_list_free (GList *list)
+{
+	if (list != NULL) {
+		g_list_foreach (list, (GFunc) file_data_unref, NULL);
+		g_list_free (list);
+	}
 }
